#ifndef MAGMA
#define MAGMA
#endif
//----------------------------------------------------------------------------
// ObjectWindows
// (C) Copyright 1991, 1994 by Borland International, All Rights Reserved
//
//   Definition of class TMagmaEdit.  This defines the basic behavior
//   of all edit controls.
//----------------------------------------------------------------------------
#if !defined(OWL_MAGMAEDIT_H)
#define OWL_MAGMAEDIT_H

#if defined(MAGMA)
#include <magmaed.h>
#endif

#if !defined(OWL_STATIC_H)
# include <owl/static.h>
#endif
#include <owl/edit.rh>

// BEH: Added new kinds of event handlers for Owner-Draw messages that 
//      honor the return value, and pass the correct structure.
#define EV_WM_MEDRAWITEM \
{WM_DRAWITEM, 0, (TAnyDispatcher)::LRESULT_WPARAM_LPARAM_Dispatch,\
   (TMyPMF)LRESULT_U_MEDRAWITEM_Sig(&TMyClass::EvMEDrawItem)}

template <class T>
inline LRESULT(T::*LRESULT_U_MEDRAWITEM_Sig(LRESULT(T::*pmf)(UINT, MEDRAWITEMSTRUCT far&)))
                                      (UINT, MEDRAWITEMSTRUCT far&)
{
  return pmf;
}

#define EV_WM_MEDELETEITEM \
{WM_DELETEITEM, 0, (TAnyDispatcher)::LRESULT_WPARAM_LPARAM_Dispatch,\
   (TMyPMF)LRESULT_U_MEDELETEITEM_Sig(&TMyClass::EvMEDeleteItem)}

template <class T>
inline LRESULT(T::*LRESULT_U_MEDELETEITEM_Sig(LRESULT(T::*pmf)(UINT, MEDELETEITEMSTRUCT far&)))
                                      (UINT, MEDELETEITEMSTRUCT far&)
{
  return pmf;
}

#define EV_WM_MEMEASUREITEM \
{WM_MEASUREITEM, 0, (TAnyDispatcher)::LRESULT_WPARAM_LPARAM_Dispatch,\
   (TMyPMF)LRESULT_U_MEMEASUREITEM_Sig(&TMyClass::EvMEMeasureItem)}

template <class T>
inline LRESULT(T::*LRESULT_U_MEMEASUREITEM_Sig(LRESULT(T::*pmf)(UINT, MEMEASUREITEMSTRUCT far&)))
                                      (UINT, MEMEASUREITEMSTRUCT far&)
{
  return pmf;
}

class _OWLCLASS TValidator;

//
//  class TMagmaEdit
//  ----- -----
//
class _OWLCLASS TMagmaEdit : public TStatic {
  public:
    TMagmaEdit(TWindow*        parent,
          int             id,
          const char far* text,
          int x, int y, int w, int h,
          uint            textLen = 0,
          bool            multiline = false,
          TModule*        module = 0);

    TMagmaEdit(TWindow*   parent,
          int        resourceId,
          uint       textLen = 0,
          TModule*   module = 0);

   ~TMagmaEdit();

    //
    // Accessing
    //
#if defined(MAGMA)
    LONG    GetNumLines() const;
    int     GetLineLength(LONG lineNumber) const;
    BOOL    GetLine(char far* str, int strSize, LONG lineNumber) const;
    void    GetSubText(char far* str, DWORD startPos, DWORD endPos) const;
    void    GetSelection(UINT& startPos, UINT& endPos) const;
    void    ExGetSelection(LPCHARRANGE lpCharRange) const;
    void    GetSelText(LPSTR lpBuf) const;
#else
    int     GetNumLines() const;
    int     GetLineLength(int lineNumber) const;
    bool    GetLine(char far* str, int strSize, int lineNumber) const;
    void    GetSubText(char far* str, uint startPos, uint endPos) const;
    void    GetSelection(uint& startPos, uint& endPos) const;
#endif

    bool    IsModified() const;
    void    ClearModify() {HandleMessage(EM_SETMODIFY);}

#if defined(MAGMA)
    LONG    GetLineFromPos(DWORD charPos) const;
    DWORD   GetLineIndex(LONG lineNumber) const;
#else
    int     GetLineFromPos(uint charPos) const;
    uint    GetLineIndex(int lineNumber) const;
#endif

    uint    Transfer(void* buffer, TTransferDirection direction);

    //
    // Lock and unlock this edit control's buffer. Allows direct access to the
    // text in the edit control.
    //
    char far* LockBuffer(uint newSize = 0);
    void    UnlockBuffer(const char far* buffer, bool updateHandle = false);

    //
    // operations
    //
#if defined(MAGMA)
    BOOL    DeleteSubText(DWORD startPos, DWORD endPos);
    BOOL    DeleteLine(LONG lineNumber);
#else
    bool    DeleteSubText(uint startPos, uint endPos);
    bool    DeleteLine(int lineNumber);
#endif
    bool    DeleteSelection();
    bool    SetSelection(uint startPos, uint endPos);
#if defined(MAGMA)
    BOOL    ExSetSelection(LPCHARRANGE lpCharRange) const;
#endif

    void    Scroll(int horizontalUnit, int verticalUnit);
    void    Insert(const char far* str);
#if defined(MAGMA)
    LONG    Search(DWORD startPos, const char far* text,
#else
    int     Search(uint startPos, const char far* text,
#endif
                   bool caseSensitive=false, bool wholeWord=false,
                   bool up=false);

    void    GetRect(TRect& frmtRect) const;
    void    SetRect(const TRect& frmtRect);
    void    SetRectNP(const TRect& frmtRect);
    void    FormatLines(bool addEOL);
    void    SetTabStops(int numTabs, const int far* tabs);

    HLOCAL  GetHandle() const;
    void    SetHandle(HLOCAL localMem);

    void    SetPasswordChar(uint ch) {HandleMessage(EM_SETPASSWORDCHAR, ch);}

#if defined(MAGMA)
    LONG    GetFirstVisibleLine() const;
#else
    int     GetFirstVisibleLine() const;
#endif
    void    SetReadOnly(bool readOnly);
    uint    GetPasswordChar() const;

    EDITWORDBREAKPROC GetWordBreakProc() const;
    void    SetWordBreakProc(EDITWORDBREAKPROC proc);

    //
    // clipboard operations
    //
    bool    CanUndo() const;
    void    EmptyUndoBuffer() {HandleMessage(EM_EMPTYUNDOBUFFER);}
    void    Undo() {HandleMessage(WM_UNDO);}
    void    Paste() {HandleMessage(WM_PASTE);}
    void    Copy() {HandleMessage(WM_COPY);}
    void    Cut() {HandleMessage(WM_CUT);}
    
    bool    IsValid(bool reportErr = false);
    void    SetValidator(TValidator* validator);
    void    ValidatorError();

    //
    // Override TStatic virtual member functions
    //
    void Clear();

  protected:
    //
    // command response functions
    //
    void    CmEditCut() {Cut();}                  // CM_EDITCUT
    void    CmEditCopy() {Copy();}                // CM_EDITCOPY
    void    CmEditPaste() {Paste();}              // CM_EDITPASTE
    void    CmEditDelete() {DeleteSelection();}   // CM_EDITDELETE
    void    CmEditClear() {Clear();}              // CM_EDITCLEAR
    void    CmEditUndo() {Undo();}                // CM_EDITUNDO

    //
    // command enabler functions
    //
    void CmSelectEnable(TCommandEnabler& commandHandler);
    void CmPasteEnable(TCommandEnabler& commandHandler);
    void CmCharsEnable(TCommandEnabler& commandHandler);
    void CmModEnable(TCommandEnabler& commandHandler);

    //
    // child id notification handled at the child
    //
    void    ENErrSpace();  // EN_ERRSPACE

    //
    // Override TWindow virtual member functions
    //
    char far* GetClassName();
    void      SetupWindow();

    //
    // Input validation object
    //
    TValidator*  Validator;

    void         EvChar(uint key, uint repeatCount, uint flags);
    void         EvKeyDown(uint key, uint repeatCount, uint flags);
    uint         EvGetDlgCode(MSG far*);
    void         EvSetFocus(HWND hWndLostFocus);
    void         EvKillFocus(HWND hWndGetFocus);
    bool         CanClose();

    //
    // handler for input validation message sent by parent
    //
    void         EvChildInvalid(HWND);

  private:
    //
    // hidden to prevent accidental copying or assignment
    //
    TMagmaEdit(const TMagmaEdit&);
    TMagmaEdit& operator =(const TMagmaEdit&);


    // Used to prevent 'oscillation' when a validated window with invalid
    // input is losing focus to another validated window with invalid input
    // Without this flag, the two windows will fight for focus
    static TMagmaEdit* ValidatorReFocus;

  DECLARE_RESPONSE_TABLE(TMagmaEdit);
  DECLARE_STREAMABLE(_OWLCLASS, TMagmaEdit, 1);
};

//
// edit control notification macros. methods are: void method()
//
// EV_EN_CHANGE(id, method)
// EV_EN_ERRSPACE(id, method)
// EV_EN_HSCROLL(id, method)
// EV_EN_KILLFOCUS(id, method)
// EV_EN_MAXTEXT(id, method)
// EV_EN_SETFOCUS(id, method)
// EV_EN_UPDATE(id, method)
// EV_EN_VSCROLL(id, method)

//----------------------------------------------------------------------------
// Inlines
//----------------------------------------------------------------------------

//
// returns the number of lines in the associated edit control
//
// note that GetNumLines returns 1 when the edit control has no text (i.e.
// it has one line with no text in it). Returns zero if an error occurs
//
#if defined(MAGMA)
inline LONG TMagmaEdit::GetNumLines() const {
  return (LONG)CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_GETLINECOUNT);
}
#else
inline int TMagmaEdit::GetNumLines() const {
  return (int)CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_GETLINECOUNT);
}
#endif

inline bool TMagmaEdit::IsModified() const {
  return (bool)CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_GETMODIFY);
}

//
// selects the characters in the range "startPos .. endPos"
//
inline bool TMagmaEdit::SetSelection(uint startPos, uint endPos) {
#if defined(BI_PLAT_WIN32)
  return HandleMessage(EM_SETSEL, startPos, endPos) != 0;
#else
  return HandleMessage(EM_SETSEL, 0, MAKELPARAM(startPos, endPos)) != 0;
#endif
}

// returns the starting and ending positions of the selected text
//
inline void TMagmaEdit::GetSelection(uint& startPos, uint& endPos) const {
#if defined(BI_PLAT_WIN32)
  CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_GETSEL, (WPARAM)&startPos, (LPARAM)&endPos);
#else
  LRESULT  retValue = CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_GETSEL);
  startPos = LOWORD(retValue);
  endPos   = HIWORD(retValue);
#endif
}

#if defined(MAGMA)
inline BOOL TMagmaEdit::ExSetSelection(LPCHARRANGE charRange) const {
    return (BOOL) CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_EXSETSEL, 0, (LPARAM) charRange) != 0;
}

inline void TMagmaEdit::ExGetSelection(LPCHARRANGE charRange) const {
    CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_EXGETSEL, 0, (LPARAM) charRange);
}

inline void TMagmaEdit::GetSelText(LPSTR lpBuf) const {
    CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_GETSELTEXT, 0, (LPARAM) lpBuf);
}
#endif


//
// returns the line number associated with character index "CharPos"
//
// if "CharPos" is greater than the number of characters, the last line is
// returned
// if "CharPos" is -1, the line containing the beginning of the selection (or
// the line containing the caret if no selection) is returned
//
#if defined(MAGMA)
inline LONG TMagmaEdit::GetLineFromPos(DWORD charPos) const {
  return (LONG)CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_LINEFROMCHAR, 0, charPos);
}
#else
inline int TMagmaEdit::GetLineFromPos(uint charPos) const {
  return (int)CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_LINEFROMCHAR, charPos);
}
#endif

//
// returns the character index of line number "LineNumber" or -1 if
// "LineNumber" is greater than the number of lines
//
// if "LineNumber" is -1, the line containing the caret is used
//
#if defined(MAGMA)
inline DWORD TMagmaEdit::GetLineIndex(LONG lineNumber) const {
  return (LONG)CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_LINEINDEX, 0, lineNumber);
}
#else
inline uint TMagmaEdit::GetLineIndex(int lineNumber) const {
  return (int)CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_LINEINDEX, lineNumber);
}
#endif

//
// scrolls the text by the specified horizontal and vertical amounts
//
inline void TMagmaEdit::Scroll(int horizUnit, int vertUnit) {
#if defined(BI_PLAT_WIN32)
  HandleMessage(EM_LINESCROLL, horizUnit, vertUnit);
#else
  HandleMessage(EM_LINESCROLL, 0, MAKELONG(vertUnit, horizUnit));
#endif
}

//
// sets the selection to the "str" (does a "paste" type of action
// without affecting the clipboard)
//
inline void TMagmaEdit::Insert(const char far* str) {
  HandleMessage(EM_REPLACESEL, 0, (LPARAM)str);
}

inline void TMagmaEdit::GetRect(TRect& frmtRect) const {
  CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_GETRECT, 0, (LPARAM)(TRect FAR*)&frmtRect);
}

inline void TMagmaEdit::SetRect(const TRect& frmtRect) {
  HandleMessage(EM_SETRECT, 0, (LPARAM)(const TRect FAR*)&frmtRect);
}

inline void TMagmaEdit::SetRectNP(const TRect& frmtRect) {
  HandleMessage(EM_SETRECTNP, 0, (LPARAM)(const TRect FAR*)&frmtRect);
}

inline void TMagmaEdit::FormatLines(bool addEOL) {
  HandleMessage(EM_FMTLINES, addEOL);
}

inline void TMagmaEdit::SetTabStops(int numTabs, const int far* tabs) {
  HandleMessage(EM_SETTABSTOPS, numTabs, (LPARAM)tabs);
}

inline HLOCAL TMagmaEdit::GetHandle() const {
  return (HLOCAL)CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_GETHANDLE);
}

inline void TMagmaEdit::SetHandle(HLOCAL localMem) {
  HandleMessage(EM_SETHANDLE, (WPARAM)localMem);
}

#if defined(MAGMA)
inline LONG TMagmaEdit::GetFirstVisibleLine() const {
  return (LONG)CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_GETFIRSTVISIBLELINE);
}
#else
inline int TMagmaEdit::GetFirstVisibleLine() const {
  return (int)CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_GETFIRSTVISIBLELINE);
}
#endif

inline void TMagmaEdit::SetReadOnly(bool readOnly) {
  HandleMessage(EM_SETREADONLY, readOnly);
}

inline uint TMagmaEdit::GetPasswordChar() const {
  return (uint)CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_GETPASSWORDCHAR);
}

inline EDITWORDBREAKPROC TMagmaEdit::GetWordBreakProc() const {
  return (EDITWORDBREAKPROC)CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_GETWORDBREAKPROC);
}

inline void TMagmaEdit::SetWordBreakProc(EDITWORDBREAKPROC proc) {
  HandleMessage(EM_SETWORDBREAKPROC, 0, (LPARAM)proc);
}

inline bool TMagmaEdit::CanUndo() const {
  return (bool)CONST_CAST(TMagmaEdit*,this)->HandleMessage(EM_CANUNDO);
}

#endif  // OWL_MAGMAEDIT_H

